=== modified file 'NEWS'
--- NEWS	2005-10-15 01:07:02 +0000
+++ NEWS	2005-10-16 05:15:58 +0000
@@ -6,6 +6,19 @@
       of revisions that are not tied to a particular internal
       representation.
 
+    * Config options have moved from bzrlib.osutils to bzrlib.config.
+      Configuration is now done via the config.Config interface:
+      Depending on whether you have a Branch, a Location or no information
+      available, construct a *Config, and use its signature_checking,
+      username and user_email methods. (Robert Collins)
+
+    * Plugins are now loaded under bzrlib.plugins, not bzrlib.plugin, and
+      they are made available for other plugins to use. You should not 
+      import other plugins during the __init__ of your plugin though, as 
+      no ordering is guaranteed, and the plugins directory is not on the
+      python path.
+
+
   IMPROVEMENTS:
 
     * The config directory is now ~/.bazaar, and there is a single file 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-15 01:07:02 +0000
+++ bzrlib/branch.py	2005-10-16 05:15:58 +0000
@@ -303,7 +303,7 @@
             return self._transport.base
         return None
 
-    base = property(_get_base)
+    base = property(_get_base, doc="The URL for the root of this branch.")
 
     def _finish_transaction(self):
         """Exit the current transaction."""

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-15 01:07:02 +0000
+++ bzrlib/builtins.py	2005-10-16 05:15:58 +0000
@@ -1099,13 +1099,14 @@
     def run(self, email=False):
         try:
             b = bzrlib.branch.Branch.open_containing('.')
+            config = bzrlib.config.BranchConfig(b)
         except NotBranchError:
-            b = None
+            config = bzrlib.config.GlobalConfig()
         
         if email:
-            print bzrlib.config.user_email(b)
+            print config.user_email()
         else:
-            print bzrlib.config.username(b)
+            print config.username()
 
 
 class cmd_selftest(Command):

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-13 07:30:57 +0000
+++ bzrlib/commit.py	2005-10-15 11:38:29 +0000
@@ -194,13 +194,14 @@
         else:
             self.timestamp = long(timestamp)
             
+        config = bzrlib.config.BranchConfig(self.branch)
         if rev_id is None:
-            self.rev_id = _gen_revision_id(self.branch, self.timestamp)
+            self.rev_id = _gen_revision_id(config, self.timestamp)
         else:
             self.rev_id = rev_id
 
         if committer is None:
-            self.committer = bzrlib.config.username(self.branch)
+            self.committer = config.username()
         else:
             assert isinstance(committer, basestring), type(committer)
             self.committer = committer
@@ -419,8 +420,8 @@
             if file_id not in self.new_inv:
                 self.reporter.deleted(self.basis_inv.id2path(file_id))
 
-def _gen_revision_id(branch, when):
+def _gen_revision_id(config, when):
     """Return new revision-id."""
-    s = '%s-%s-' % (bzrlib.config.user_email(branch), compact_date(when))
+    s = '%s-%s-' % (config.user_email(), compact_date(when))
     s += hexlify(rand_bytes(8))
     return s

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2005-10-11 07:21:00 +0000
+++ bzrlib/config.py	2005-10-15 09:19:08 +0000
@@ -15,14 +15,284 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Configuration that affects the behaviour of Bazaar."""
+"""Configuration that affects the behaviour of Bazaar.
+
+Currently this configuration resides in ~/.bazaar/bazaar.conf
+and ~/.bazaar/branches.conf, which is written to by bzr.
+
+In bazaar.config the following options may be set:
+[DEFAULT]
+editor=name-of-program
+email=Your Name <your@email.address>
+check_signatures=require|ignore|check-available(default)
+create_signatures=always|never|when-required(default)
+
+in branches.conf, you specify the url of a branch and options for it.
+Wildcards may be used - * and ? as normal in shell completion. Options
+set in both bazaar.conf and branches.conf are overriden by the branches.conf
+setting.
+[/home/robertc/source]
+recurse=False|True(default)
+email= as above
+check_signatures= as abive 
+create_signatures= as above.
+
+explanation of options
+----------------------
+editor - this option sets the pop up editor to use during commits.
+email - this option sets the user id bzr will use when committing.
+check_signatures - this option controls whether bzr will require good gpg
+                   signatures, ignore them, or check them if they are 
+                   present.
+create_signatures - this option controls whether bzr will always create 
+                    gpg signatures, never create them, or create them if the
+                    branch is configured to require them.
+                    NB: This option is planned, but not implemented yet.
+"""
 
 from ConfigParser import ConfigParser
 import os
+from fnmatch import fnmatch
 import errno
 import re
 
 import bzrlib
+import bzrlib.errors as errors
+
+
+CHECK_IF_POSSIBLE=0
+CHECK_ALWAYS=1
+CHECK_NEVER=2
+
+
+class Config(object):
+    """A configuration policy - what username, editor, gpg needs etc."""
+
+    def get_editor(self):
+        """Get the users pop up editor."""
+        raise NotImplementedError
+
+    def _get_signature_checking(self):
+        """Template method to override signature checking policy."""
+
+    def __init__(self):
+        super(Config, self).__init__()
+
+    def user_email(self):
+        """Return just the email component of a username."""
+        e = self.username()
+        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
+        if not m:
+            raise BzrError("%r doesn't seem to contain "
+                           "a reasonable email address" % e)
+        return m.group(0)
+
+    def username(self):
+        """Return email-style username.
+    
+        Something similar to 'Martin Pool <mbp@sourcefrog.net>'
+        
+        $BZREMAIL can be set to override this, then
+        the concrete policy type is checked, and finally
+        $EMAIL is examinged.
+        but if none is found, a reasonable default is (hopefully)
+        created.
+    
+        TODO: Check it's reasonably well-formed.
+        """
+        v = os.environ.get('BZREMAIL')
+        if v:
+            return v.decode(bzrlib.user_encoding)
+    
+        v = self._get_user_id()
+        if v:
+            return v
+        
+        v = os.environ.get('EMAIL')
+        if v:
+            return v.decode(bzrlib.user_encoding)
+
+        name, email = _auto_user_id()
+        if name:
+            return '%s <%s>' % (name, email)
+        else:
+            return email
+
+    def signature_checking(self):
+        """What is the current policy for signature checking?."""
+        policy = self._get_signature_checking()
+        if policy is not None:
+            return policy
+        return CHECK_IF_POSSIBLE
+
+    def signature_needed(self):
+        """Is a signature needed when committing ?."""
+        policy = self._get_signature_checking()
+        if policy == CHECK_ALWAYS:
+            return True
+        return False
+
+
+class IniBasedConfig(Config):
+    """A configuration policy that draws from ini files."""
+
+    def _get_parser(self, file=None):
+        if self._parser is not None:
+            return self._parser
+        parser = ConfigParser()
+        if file is not None:
+            parser.readfp(file)
+        else:
+            parser.read([self._get_filename()])
+        self._parser = parser
+        return parser
+
+    def _get_section(self):
+        """Override this to define the section used by the config."""
+        return "DEFAULT"
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        section = self._get_section()
+        if section is None:
+            return None
+        if self._get_parser().has_option(section, 'check_signatures'):
+            return self._string_to_signature_policy(
+                self._get_parser().get(section, 'check_signatures'))
+
+    def _get_user_id(self):
+        """Get the user id from the 'email' key in the current section."""
+        section = self._get_section()
+        if section is not None:
+            if self._get_parser().has_option(section, 'email'):
+                return self._get_parser().get(section, 'email')
+
+    def __init__(self, get_filename):
+        super(IniBasedConfig, self).__init__()
+        self._get_filename = get_filename
+        self._parser = None
+
+    def _string_to_signature_policy(self, signature_string):
+        """Convert a string to a signing policy."""
+        if signature_string.lower() == 'check-available':
+            return CHECK_IF_POSSIBLE
+        if signature_string.lower() == 'ignore':
+            return CHECK_NEVER
+        if signature_string.lower() == 'require':
+            return CHECK_ALWAYS
+        raise errors.BzrError("Invalid signatures policy '%s'"
+                              % signature_string)
+
+
+class GlobalConfig(IniBasedConfig):
+    """The configuration that should be used for a specific location."""
+
+    def get_editor(self):
+        if self._get_parser().has_option(self._get_section(), 'editor'):
+            return self._get_parser().get(self._get_section(), 'editor')
+
+    def __init__(self):
+        super(GlobalConfig, self).__init__(config_filename)
+
+
+class LocationConfig(IniBasedConfig):
+    """A configuration object that gives the policy for a location."""
+
+    def __init__(self, location):
+        super(LocationConfig, self).__init__(branches_config_filename)
+        self._global_config = None
+        self.location = location
+
+    def _get_global_config(self):
+        if self._global_config is None:
+            self._global_config = GlobalConfig()
+        return self._global_config
+
+    def _get_section(self):
+        """Get the section we should look in for config items.
+
+        Returns None if none exists. 
+        TODO: perhaps return a NullSection that thunks through to the 
+              global config.
+        """
+        sections = self._get_parser().sections()
+        location_names = self.location.split('/')
+        if self.location.endswith('/'):
+            del location_names[-1]
+        matches=[]
+        for section in sections:
+            section_names = section.split('/')
+            if section.endswith('/'):
+                del section_names[-1]
+            names = zip(location_names, section_names)
+            matched = True
+            for name in names:
+                if not fnmatch(name[0], name[1]):
+                    matched = False
+                    break
+            if not matched:
+                continue
+            # so, for the common prefix they matched.
+            # if section is longer, no match.
+            if len(section_names) > len(location_names):
+                continue
+            # if path is longer, and recurse is not true, no match
+            if len(section_names) < len(location_names):
+                if (self._get_parser().has_option(section, 'recurse')
+                    and not self._get_parser().getboolean(section, 'recurse')):
+                    continue
+            matches.append((len(section_names), section))
+        if not len(matches):
+            return None
+        matches.sort(reverse=True)
+        return matches[0][1]
+
+    def _get_user_id(self):
+        user_id = super(LocationConfig, self)._get_user_id()
+        if user_id is not None:
+            return user_id
+        return self._get_global_config()._get_user_id()
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        check = super(LocationConfig, self)._get_signature_checking()
+        if check is not None:
+            return check
+        return self._get_global_config()._get_signature_checking()
+
+
+class BranchConfig(Config):
+    """A configuration object giving the policy for a branch."""
+
+    def _get_location_config(self):
+        if self._location_config is None:
+            self._location_config = LocationConfig(self.branch.base)
+        return self._location_config
+
+    def _get_user_id(self):
+        """Return the full user id for the branch.
+    
+        e.g. "John Hacker <jhacker@foo.org>"
+        This is looked up in the email controlfile for the branch.
+        """
+        try:
+            return (self.branch.controlfile("email", "r") 
+                    .read()
+                    .decode(bzrlib.user_encoding)
+                    .rstrip("\r\n"))
+        except errors.NoSuchFile, e:
+            pass
+        
+        return self._get_location_config()._get_user_id()
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        return self._get_location_config()._get_signature_checking()
+
+    def __init__(self, branch):
+        super(BranchConfig, self).__init__()
+        self._location_config = None
+        self.branch = branch
 
 
 def config_dir():
@@ -40,65 +310,9 @@
     return os.path.join(config_dir(), 'bazaar.conf')
 
 
-def _get_config_parser(file=None):
-    parser = ConfigParser()
-    if file is not None:
-        parser.readfp(file)
-    else:
-        parser.read([config_filename()])
-    return parser
-
-
-def get_editor(parser=None):
-    if parser is None:
-        parser = _get_config_parser()
-    if parser.has_option('DEFAULT', 'editor'):
-        return parser.get('DEFAULT', 'editor')
-
-
-def _get_user_id(branch=None, parser = None):
-    """Return the full user id from a file or environment variable.
-
-    e.g. "John Hacker <jhacker@foo.org>"
-
-    branch
-        A branch to use for a per-branch configuration, or None.
-
-    The following are searched in order:
-
-    1. $BZREMAIL
-    2. .bzr/email for this branch.
-    3. ~/.bzr.conf/email
-    4. $EMAIL
-    """
-    v = os.environ.get('BZREMAIL')
-    if v:
-        return v.decode(bzrlib.user_encoding)
-
-    if branch:
-        try:
-            return (branch.controlfile("email", "r") 
-                    .read()
-                    .decode(bzrlib.user_encoding)
-                    .rstrip("\r\n"))
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-        except BzrError, e:
-            pass
-    
-    if parser is None:
-        parser = _get_config_parser()
-    if parser.has_option('DEFAULT', 'email'):
-        email = parser.get('DEFAULT', 'email')
-        if email is not None:
-            return email
-
-    v = os.environ.get('EMAIL')
-    if v:
-        return v.decode(bzrlib.user_encoding)
-    else:    
-        return None
+def branches_config_filename():
+    """Return per-user configuration ini file filename."""
+    return os.path.join(config_dir(), 'branches.conf')
 
 
 def _auto_user_id():
@@ -137,34 +351,3 @@
     return realname, (username + '@' + socket.gethostname())
 
 
-def username(branch):
-    """Return email-style username.
-
-    Something similar to 'Martin Pool <mbp@sourcefrog.net>'
-
-    TODO: Check it's reasonably well-formed.
-    """
-    v = _get_user_id(branch)
-    if v:
-        return v
-    
-    name, email = _auto_user_id()
-    if name:
-        return '%s <%s>' % (name, email)
-    else:
-        return email
-
-
-def user_email(branch):
-    """Return just the email component of a username."""
-    e = _get_user_id(branch)
-    if e:
-        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
-        if not m:
-            raise BzrError("%r doesn't seem to contain "
-                           "a reasonable email address" % e)
-        return m.group(0)
-
-    return _auto_user_id()[1]
-
-

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2005-10-11 07:21:00 +0000
+++ bzrlib/msgeditor.py	2005-10-14 01:56:08 +0000
@@ -32,7 +32,7 @@
     except KeyError:
         pass
 
-    e = config.get_editor()
+    e = config.GlobalConfig().get_editor()
     if e is not None:
         yield e
         

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-10-15 01:07:02 +0000
+++ bzrlib/osutils.py	2005-10-16 05:15:58 +0000
@@ -30,7 +30,6 @@
 import types
 
 import bzrlib
-from bzrlib.config import config_dir, _get_user_id
 from bzrlib.errors import BzrError
 from bzrlib.trace import mutter
 

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2005-10-11 04:32:38 +0000
+++ bzrlib/plugin.py	2005-10-16 00:22:17 +0000
@@ -24,6 +24,7 @@
 
 
 import os
+import bzrlib
 from bzrlib.config import config_dir
 DEFAULT_PLUGIN_PATH = os.path.join(config_dir(), 'plugins')
 
@@ -110,9 +111,10 @@
                 plugin_info = imp.find_module(name, [d])
                 mutter('load plugin %r' % (plugin_info,))
                 try:
-                    plugin = imp.load_module('bzrlib.plugin.' + name,
+                    plugin = imp.load_module('bzrlib.plugins.' + name,
                                              *plugin_info)
                     all_plugins.append(plugin)
+                    setattr(bzrlib.plugins, name, plugin)
                 finally:
                     if plugin_info[0] is not None:
                         plugin_info[0].close()

=== modified file 'bzrlib/selftest/testconfig.py'
--- bzrlib/selftest/testconfig.py	2005-10-13 04:26:42 +0000
+++ bzrlib/selftest/testconfig.py	2005-10-15 09:19:08 +0000
@@ -24,6 +24,7 @@
 
 #import bzrlib specific imports here
 import bzrlib.config as config
+import bzrlib.errors as errors
 from bzrlib.selftest import TestCase, TestCaseInTempDir
 
 
@@ -32,6 +33,39 @@
                       "editor=vim\n"
                       "gpg_signing_command=gnome-gpg\n")
 
+
+sample_always_signatures = ("[DEFAULT]\n"
+                            "check_signatures=require\n")
+
+
+sample_ignore_signatures = ("[DEFAULT]\n"
+                            "check_signatures=ignore\n")
+
+
+sample_maybe_signatures = ("[DEFAULT]\n"
+                            "check_signatures=check-available\n")
+
+
+sample_branches_text = ("[http://www.example.com]\n"
+                        "# Top level policy\n"
+                        "email=Robert Collins <robertc@example.org>\n"
+                        "[http://www.example.com/useglobal]\n"
+                        "# different project, forces global lookup\n"
+                        "recurse=false\n"
+                        "[/b/]\n"
+                        "check_signatures=require\n"
+                        "# test trailing / matching with no children\n"
+                        "[/a/]\n"
+                        "check_signatures=check-available\n"
+                        "# test trailing / matching\n"
+                        "[/a/*]\n"
+                        "#subdirs will match but not the parent\n"
+                        "recurse=False\n"
+                        "[/a/c]\n"
+                        "check_signatures=ignore\n"
+                        "#testing explicit beats globs\n")
+
+
 class InstrumentedConfigParser(object):
     """A config parser look-enough-alike to record calls made to it."""
 
@@ -42,6 +76,74 @@
         self._calls.append(('read', filenames))
 
 
+class FakeBranch(object):
+
+    def __init__(self):
+        self.base = "http://example.com/branches/demo"
+        self.email = 'Robert Collins <robertc@example.net>\n'
+
+    def controlfile(self, filename, mode):
+        if filename != 'email':
+            raise NotImplementedError
+        if self.email is not None:
+            return StringIO(self.email)
+        raise errors.NoSuchFile
+
+
+class InstrumentedConfig(config.Config):
+    """An instrumented config that supplies stubs for template methods."""
+    
+    def __init__(self):
+        super(InstrumentedConfig, self).__init__()
+        self._calls = []
+        self._signatures = config.CHECK_NEVER
+
+    def _get_user_id(self):
+        self._calls.append('_get_user_id')
+        return "Robert Collins <robert.collins@example.org>"
+
+    def _get_signature_checking(self):
+        self._calls.append('_get_signature_checking')
+        return self._signatures
+
+
+class TestConfig(TestCase):
+
+    def test_constructs(self):
+        config.Config()
+ 
+    def test_no_default_editor(self):
+        self.assertRaises(NotImplementedError, config.Config().get_editor)
+
+    def test_user_email(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual('robert.collins@example.org', my_config.user_email())
+        self.assertEqual(['_get_user_id'], my_config._calls)
+
+    def test_username(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual('Robert Collins <robert.collins@example.org>',
+                         my_config.username())
+        self.assertEqual(['_get_user_id'], my_config._calls)
+
+    def test_signatures_default(self):
+        my_config = config.Config()
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+
+    def test_signatures_template_method(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
+        self.assertEqual(['_get_signature_checking'], my_config._calls)
+
+    def test_signatures_template_method_none(self):
+        my_config = InstrumentedConfig()
+        my_config._signatures = None
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+        self.assertEqual(['_get_signature_checking'], my_config._calls)
+
+
 class TestConfigPath(TestCase):
 
     def setUp(self):
@@ -59,28 +161,63 @@
         self.assertEqual(config.config_filename(),
                          '/home/bogus/.bazaar/bazaar.conf')
 
-
-class TestGetConfig(TestCase):
+    def test_branches_config_filename(self):
+        self.assertEqual(config.branches_config_filename(),
+                         '/home/bogus/.bazaar/branches.conf')
+
+class TestIniConfig(TestCase):
+
+    def test_contructs(self):
+        my_config = config.IniBasedConfig("nothing")
 
     def test_from_fp(self):
         config_file = StringIO(sample_config_text)
-        self.failUnless(isinstance(config._get_config_parser(file=config_file),
+        my_config = config.IniBasedConfig(None)
+        self.failUnless(
+            isinstance(my_config._get_parser(file=config_file),
                         ConfigParser))
 
+    def test_cached(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.IniBasedConfig(None)
+        parser = my_config._get_parser(file=config_file)
+        self.failUnless(my_config._get_parser() is parser)
+
+
+
+class TestGetConfig(TestCase):
+
+    def test_constructs(self):
+        my_config = config.GlobalConfig()
+
     def test_calls_read_filenames(self):
-        # note the monkey patching. if config access was via a class instance,
-        # we would not have to - if this changes in future, be sure to stop 
-        # monkey patching RBC 20051011
+        # replace the class that is constructured, to check its parameters
         oldparserclass = config.ConfigParser
         config.ConfigParser = InstrumentedConfigParser
+        my_config = config.GlobalConfig()
         try:
-            parser = config._get_config_parser()
+            parser = my_config._get_parser()
         finally:
             config.ConfigParser = oldparserclass
         self.failUnless(isinstance(parser, InstrumentedConfigParser))
         self.assertEqual(parser._calls, [('read', [config.config_filename()])])
 
 
+class TestBranchConfig(TestCaseInTempDir):
+
+    def test_constructs(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertRaises(TypeError, config.BranchConfig)
+
+    def test_get_location_config(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        location_config = my_config._get_location_config()
+        self.assertEqual(branch.base, location_config.location)
+        self.failUnless(location_config is my_config._get_location_config())
+
+
 class TestConfigItems(TestCase):
 
     def setUp(self):
@@ -96,25 +233,214 @@
 
     def tearDown(self):
         os.environ['HOME'] = self.oldenv
+        if os.environ.get('BZREMAIL') is not None:
+            del os.environ['BZREMAIL']
         if self.bzr_email is not None:
             os.environ['BZREMAIL'] = self.bzr_email
         if self.email is not None:
             os.environ['EMAIL'] = self.email
         super(TestConfigItems, self).tearDown()
 
+
+class TestGlobalConfigItems(TestConfigItems):
+
     def test_user_id(self):
         config_file = StringIO(sample_config_text)
-        parser = config._get_config_parser(file=config_file)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
         self.assertEqual("Robert Collins <robertc@example.com>",
-                         config._get_user_id(parser = parser))
+                         my_config._get_user_id())
 
     def test_absent_user_id(self):
         config_file = StringIO("")
-        parser = config._get_config_parser(file=config_file)
-        self.assertEqual(None,
-                         config._get_user_id(parser = parser))
-
-    def test_configured_edit(self):
-        config_file = StringIO(sample_config_text)
-        parser = config._get_config_parser(file=config_file)
-        self.assertEqual("vim", config.get_editor(parser = parser))
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(None, my_config._get_user_id())
+
+    def test_configured_editor(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual("vim", my_config.get_editor())
+
+    def test_signatures_always(self):
+        config_file = StringIO(sample_always_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_ALWAYS,
+                         my_config.signature_checking())
+        self.assertEqual(True, my_config.signature_needed())
+
+    def test_signatures_if_possible(self):
+        config_file = StringIO(sample_maybe_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def test_signatures_ignore(self):
+        config_file = StringIO(sample_ignore_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_NEVER,
+                         my_config.signature_checking())
+        self.assertEqual(False, my_config.signature_needed())
+
+
+class TestLocationConfig(TestConfigItems):
+
+    def test_constructs(self):
+        my_config = config.LocationConfig('http://example.com')
+        self.assertRaises(TypeError, config.LocationConfig)
+
+    def test_branch_calls_read_filenames(self):
+        # replace the class that is constructured, to check its parameters
+        oldparserclass = config.ConfigParser
+        config.ConfigParser = InstrumentedConfigParser
+        my_config = config.LocationConfig('http://www.example.com')
+        try:
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigParser = oldparserclass
+        self.failUnless(isinstance(parser, InstrumentedConfigParser))
+        self.assertEqual(parser._calls, [('read', [config.branches_config_filename()])])
+
+    def test_get_global_config(self):
+        my_config = config.LocationConfig('http://example.com')
+        global_config = my_config._get_global_config()
+        self.failUnless(isinstance(global_config, config.GlobalConfig))
+        self.failUnless(global_config is my_config._get_global_config())
+
+    def test__get_section_no_match(self):
+        self.get_location_config('/')
+        self.assertEqual(None, self.my_config._get_section())
+        
+    def test__get_section_exact(self):
+        self.get_location_config('http://www.example.com')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+   
+    def test__get_section_suffix_does_not(self):
+        self.get_location_config('http://www.example.com-com')
+        self.assertEqual(None, self.my_config._get_section())
+
+    def test__get_section_subdir_recursive(self):
+        self.get_location_config('http://www.example.com/com')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_matches(self):
+        self.get_location_config('http://www.example.com/useglobal')
+        self.assertEqual('http://www.example.com/useglobal',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_nonrecursive(self):
+        self.get_location_config(
+            'http://www.example.com/useglobal/childbranch')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_trailing_slash(self):
+        self.get_location_config('/b')
+        self.assertEqual('/b/', self.my_config._get_section())
+
+    def test__get_section_subdir_child(self):
+        self.get_location_config('/a/foo')
+        self.assertEqual('/a/*', self.my_config._get_section())
+
+    def test__get_section_subdir_child_child(self):
+        self.get_location_config('/a/foo/bar')
+        self.assertEqual('/a/', self.my_config._get_section())
+
+    def test__get_section_trailing_slash_with_children(self):
+        self.get_location_config('/a/')
+        self.assertEqual('/a/', self.my_config._get_section())
+
+    def test__get_section_explicit_over_glob(self):
+        self.get_location_config('/a/c')
+        self.assertEqual('/a/c', self.my_config._get_section())
+
+    def get_location_config(self, location, global_config=None):
+        if global_config is None:
+            global_file = StringIO(sample_config_text)
+        else:
+            global_file = StringIO(global_config)
+        branches_file = StringIO(sample_branches_text)
+        self.my_config = config.LocationConfig(location)
+        self.my_config._get_parser(branches_file)
+        self.my_config._get_global_config()._get_parser(global_file)
+
+    def test_location_without_username(self):
+        self.get_location_config('http://www.example.com/useglobal')
+        self.assertEqual('Robert Collins <robertc@example.com>',
+                         self.my_config.username())
+
+    def test_location_not_listed(self):
+        self.get_location_config('/home/robertc/sources')
+        self.assertEqual('Robert Collins <robertc@example.com>',
+                         self.my_config.username())
+
+    def test_overriding_location(self):
+        self.get_location_config('http://www.example.com/foo')
+        self.assertEqual('Robert Collins <robertc@example.org>',
+                         self.my_config.username())
+
+    def test_signatures_not_set(self):
+        self.get_location_config('http://www.example.com',
+                                 global_config=sample_ignore_signatures)
+        self.assertEqual(config.CHECK_NEVER,
+                         self.my_config.signature_checking())
+
+    def test_signatures_never(self):
+        self.get_location_config('/a/c')
+        self.assertEqual(config.CHECK_NEVER,
+                         self.my_config.signature_checking())
+        
+    def test_signatures_when_available(self):
+        self.get_location_config('/a/', global_config=sample_ignore_signatures)
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         self.my_config.signature_checking())
+        
+    def test_signatures_always(self):
+        self.get_location_config('/b')
+        self.assertEqual(config.CHECK_ALWAYS,
+                         self.my_config.signature_checking())
+        
+
+class TestBranchConfigItems(TestConfigItems):
+
+    def test_user_id(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertEqual("Robert Collins <robertc@example.net>",
+                         my_config._get_user_id())
+        branch.email = "John"
+        self.assertEqual("John", my_config._get_user_id())
+
+    def test_not_set_in_branch(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        branch.email = None
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual("Robert Collins <robertc@example.com>",
+                         my_config._get_user_id())
+        branch.email = "John"
+        self.assertEqual("John", my_config._get_user_id())
+
+    def test_BZREMAIL_OVERRIDES(self):
+        os.environ['BZREMAIL'] = "Robert Collins <robertc@example.org>"
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertEqual("Robert Collins <robertc@example.org>",
+                         my_config.username())
+    
+    def test_signatures_forced(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_always_signatures)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual(config.CHECK_ALWAYS, my_config.signature_checking())

